import matplotlib.pyplot as plt
from mpl_toolkits.mplot3d import Axes3D 
import numpy as np
import datetime
from seqreppy.config import default_results_img
from seqreppy.view.mplconfig import mplconfig 


def make_1D(seq_encoded, model_signature):
	config = mplconfig.get(model_signature)
	pltconfig = config.get("pltconfig")
	plt.clf()
	plt.plot(seq_encoded, config.get("marker"), **config.get("plotconfig"))
	plt.xlabel(pltconfig.get("xlabel"))
	plt.ylabel(pltconfig.get("ylabel"))
	plt.ticklabel_format(style='sci', scilimits=(10,1))
	plt.tight_layout() 
	return plt
	
def make_2D(seq_encoded, model_signature):
	config = mplconfig.get(model_signature)
	pltconfig = config.get("pltconfig")
	plt.clf()
	plt.plot(seq_encoded[0], seq_encoded[1], config.get("marker"), **config.get("plotconfig"))
	plt.xlabel(pltconfig.get("xlabel"))
	plt.ylabel(pltconfig.get("ylabel"))
	plt.ticklabel_format(style='sci', scilimits=(10,1))
	plt.tight_layout()
	return plt
	
def make_3D(seq_encoded, model_signature):
	config = mplconfig.get(model_signature)
	pltconfig = config.get("pltconfig")
	ax3d = Axes3D(plt.figure())	
	ax3d.plot(xs=seq_encoded[0], ys=seq_encoded[1], zs=seq_encoded[2], zdir='z', **config.get("plotconfig"))
	ax3d.set_xlabel(pltconfig.get("xlabel"))
	ax3d.set_ylabel(pltconfig.get("ylabel"))
	ax3d.set_zlabel(pltconfig.get("zlabel"))
	ax3d.ticklabel_format(style='sci', scilimits=(10,1))
	return plt

def build_power_spectrum(model_signature, power_spectrum, kargs):
	start = kargs.get("start")
	if(not start): start = 1
	end = kargs.get("end")
	config = mplconfig.get(model_signature)
	pltconfig = config.get("pltconfig")
	plt.clf()
	plt.plot(power_spectrum[start:end], config.get("marker"), **config.get("plotconfig"))
	plt.xlabel(pltconfig.get("xlabel"))
	plt.ylabel(pltconfig.get("ylabel"))
	plt.ticklabel_format(style='sci', scilimits=(10,1))
	plt.tight_layout()
	return plt


def fname_verify(img_kargs):
	if "fname" not in img_kargs:
		img_kargs["fname"] = ''.join(( 
									 "r_{}_".format(datetime.datetime.now().day),
									 "{}_".format(datetime.datetime.now().hour),
									 "{}_".format(datetime.datetime.now().minute),
									 "{}.png".format(datetime.datetime.now().microsecond)
								))


def visualize_power_spectrum(model_signature, power_spectrum, **kargs):
	build_power_spectrum(model_signature, power_spectrum, kargs).show()

def save_power_spectrum(model_signature, power_spectrum, **img_kargs):
	fname_verify(img_kargs)
	build_power_spectrum(model_signature, power_spectrum).savefig(**img_kargs)

def visualize(model_signature, seq_encoded):
	if(type(seq_encoded) != np.ndarray): seq_encoded = np.array(seq_encoded)
	num_axis_zero = np.ma.size(seq_encoded, axis=0)
	if seq_encoded.ndim == 1:
		make_1D(seq_encoded, model_signature).show()
	elif num_axis_zero == 2:
		make_2D(seq_encoded, model_signature).show()
	elif num_axis_zero == 3:
		make_3D(seq_encoded, model_signature).show()

def save_figure(model_signature, seq_encoded, **img_kargs):
	if(type(seq_encoded) != np.ndarray): seq_encoded = np.array(seq_encoded)
	fname_verify(img_kargs)
	num_axis_zero = np.ma.size(seq_encoded, axis=0)
	try:
		if seq_encoded.ndim == 1:
			make_1D(seq_encoded, model_signature).savefig(**img_kargs)
		elif num_axis_zero == 2:
			make_2D(seq_encoded, model_signature).savefig(**img_kargs)
		elif num_axis_zero == 3:
			make_3D(seq_encoded, model_signature).savefig(**img_kargs)
	except Exception as e: raise ViewExc(type(e).__name__)




import seqrepc as core

seqs_info = ()
seqs_data = ()
full_result = ()
source_directory = ""

def collect_fasta(source):
	global seqs_data, seqs_info
	seqs_data, seqs_info = core.collect_fasta(source);

def map_info_to_bytes(i): return bytes(seqs_info[i], "utf-8") 
 
def save_results(results, *dsts):
	mp_signatures = tuple(results.keys())
	num_seqs = len(results[mp_signatures[0]])
	assert len(mp_signatures) == len(dsts)
	for mp_signature, dst in zip(mp_signatures, dsts):
		core.store(results[mp_signature], tuple(map(map_info_to_bytes, range(num_seqs))), mp_signature, dst)

def map_full_result(mp_signature, source):
	return core.collect_encodings(mp_signature, source)

def map_simplify_result(i): return full_result[i][0]

def get_results(dict_results):
	global full_result
	mp_signatures = dict_results.keys()
	full_result = tuple(map(map_full_result, mp_signatures, dict_results.values()))
	simplified_results = tuple(map(map_simplify_result, range(len(full_result))))
	return dict(zip(mp_signatures, simplified_results))

def encode_from_file(mp_signatures, source):
	if(source_directory != source): collect_fasta(source)
	if((type(mp_signatures) != str)):
		results = tuple(tuple(core.encode(mp_signature, ''.join(seq)) for seq in seqs_data) for mp_signature in mp_signatures)
		return dict(zip(mp_signatures, results))
	else:
		results = tuple(core.encode(mp_signatures, ''.join(seq)) for seq in seqs_data)
		return {mp_signatures: results}

def encode_from_strings(mp_signatures, strings):
	if(len(strings) > 0):
		if((type(mp_signatures) != str)):
			results = tuple(tuple(core.encode(mp_signature, seq) for seq in strings) for mp_signature in mp_signatures)
			return dict(zip(mp_signatures, results))
		else:
			results = tuple(core.encode(mp_signatures, seq) for seq in strings)
			return {mp_signatures: results}



	
# i supose that the user will not save, again, the results that is already on disk. If he do this thing, we got an arror on get results. 
# get_results is similar to encode_from_*. She think only on seqs_encoded, not in seqs_info. If the user wants seqs_info,
# he can see full_result


import matplotlib as mpl
mpl.rcParams['agg.path.chunksize'] = 10000

mplconfig = {
	"atomic": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"cgr": {'pltconfig': {"xlabel": "x", "ylabel": 'y'}, 'plotconfig':{'c': "black", "markersize": 1}, 'marker':'.'},
	"icgr": {'pltconfig': {"xlabel": "x", "ylabel": 'y'}, 'plotconfig':{'c': "black", "markersize": 0.1}, 'marker':'-'},
	"imaginary": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.5}, 'marker':'-'},
	"dna_walk": {'pltconfig': {"xlabel": "Distância Nucleotídica 'L'", "ylabel": 'Dna Walk (L)'}, 'plotconfig':{'c': "black", "markersize": 0.1}, 'marker':'-'},
	"liao": {'pltconfig': {"xlabel": "x", "ylabel": 'y'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"eiip": {'pltconfig': {"xlabel": "Frequência", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.1}, 'marker':'-'},
	"binary2b": {'pltconfig': {"xlabel": "Frequência", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"binary4b": {'pltconfig': {"xlabel": "Frequência", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"integer": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"molecular_mass": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.1}, 'marker':'-'},
	"PairedNumericMapping": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"real": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"tetrahedron": {'pltconfig': {"xlabel": "x", "ylabel": 'y', "zlabel": 'z'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"voss": {'pltconfig': {"xlabel": "Frequency", "ylabel": 'Power Spectrum'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'},
	"zcurve": {'pltconfig': {"xlabel": "x", "ylabel": 'y', "zlabel": 'z'}, 'plotconfig':{'c': "black", "markersize": 0.3}, 'marker':'-'}	
}


import numpy as np

def map_axis(axis): return np.abs(np.fft.fft(axis))**2 

def power_spectrum(encoded_sequence):
	encoded_sequence = np.array(encoded_sequence)
	if encoded_sequence.ndim == 1: return np.abs(np.fft.fft(encoded_sequence))**2 
	return np.array(tuple(map(map_axis, encoded_sequence)))

def spectral_content(encoded_sequence): 
	return np.sum(power_spectrum(encoded_sequence), axis=0)

def apply_gsp(gsp_function, results):
	values = tuple(results.values())
	gsps = np.array( tuple(tuple(map(gsp_function, values[i])) for i in range(len(values)))  )
	return dict(zip(results.keys(), gsps ))


from setuptools import setup, find_packages

setup(name="seqreppy",
      version="1.0.2",
      description="Python library for numerical representations of genomic sequences",
      long_description=open("README.txt").read() + "\n\n",
      url="https://github.com/ednilsonlomazi/seqreppy",
      author="Ednilson Lomazi",
      author_email="ednilson.lomazi.gt@gmail.com",
      license="BSD 3-Clause License",
      packages=find_packages(),
)
